@@ -3,11 +3,75 @@
 
 //! Index for run IDs enabling efficient prefix lookup and shortest unique prefix computation.
 
-use super::store::RecordedRunInfo;
-use crate::errors::InvalidRunIdSelector;
+use super::{format::has_zip_extension, store::RecordedRunInfo};
+use crate::errors::{InvalidRunIdOrRecordingSelector, InvalidRunIdSelector};
+use camino::Utf8PathBuf;
 use quick_junit::ReportUuid;
 use std::{fmt, str::FromStr};
 
+/// Selector that can be either a run ID (for store lookup) or a recording path.
+///
+/// Used by commands that can consume recorded runs from either source, such as
+/// `store info`, `replay`, and `run --rerun`.
+///
+/// When parsing from a string:
+/// - Paths ending in `.zip` are treated as recording paths.
+/// - Strings containing path separators (`/` or `\`) are treated as recording
+///   paths. This enables process substitution paths like `/proc/self/fd/11`
+///   (Linux) or `/dev/fd/5` (macOS).
+/// - Everything else is parsed as a [`RunIdSelector`].
+#[derive(Clone, Debug, PartialEq, Eq)]
+pub enum RunIdOrRecordingSelector {
+    /// Select from the run store.
+    RunId(RunIdSelector),
+    /// Read from a portable recording file.
+    RecordingPath(Utf8PathBuf),
+}
+
+impl FromStr for RunIdOrRecordingSelector {
+    type Err = InvalidRunIdOrRecordingSelector;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        let path = Utf8PathBuf::from(s);
+        if has_zip_extension(&path) {
+            return Ok(RunIdOrRecordingSelector::RecordingPath(path));
+        }
+
+        match s.parse::<RunIdSelector>() {
+            Ok(selector) => Ok(RunIdOrRecordingSelector::RunId(selector)),
+            Err(_) => {
+                // If the input contains path separators, treat it as a file
+                // path. This handles process substitution paths like
+                // `/proc/self/fd/11` or `/dev/fd/5`, as well as relative paths
+                // like `./recording` or `../path/to/file`. Valid run ID
+                // selectors never contain `/` or `\`, so this is unambiguous.
+                if s.contains('/') || s.contains(std::path::MAIN_SEPARATOR) {
+                    Ok(RunIdOrRecordingSelector::RecordingPath(path))
+                } else {
+                    Err(InvalidRunIdOrRecordingSelector {
+                        input: s.to_owned(),
+                    })
+                }
+            }
+        }
+    }
+}
+
+impl Default for RunIdOrRecordingSelector {
+    fn default() -> Self {
+        RunIdOrRecordingSelector::RunId(RunIdSelector::Latest)
+    }
+}
+
+impl fmt::Display for RunIdOrRecordingSelector {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            RunIdOrRecordingSelector::RunId(selector) => write!(f, "{selector}"),
+            RunIdOrRecordingSelector::RecordingPath(path) => write!(f, "{path}"),
+        }
+    }
+}
+
 /// Selector for identifying a run, either the most recent or by prefix.
 ///
 /// This is used by CLI commands that need to specify a run ID. The `Latest`
@@ -285,7 +349,7 @@ pub enum PrefixResolutionError {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::record::{RecordedRunStatus, RecordedSizes, format::RECORD_FORMAT_VERSION};
+    use crate::record::{RecordedRunStatus, RecordedSizes, format::STORE_FORMAT_VERSION};
     use chrono::TimeZone;
     use semver::Version;
     use std::collections::BTreeMap;
@@ -298,7 +362,7 @@ mod tests {
             .unwrap();
         RecordedRunInfo {
             run_id,
-            store_format_version: RECORD_FORMAT_VERSION,
+            store_format_version: STORE_FORMAT_VERSION,
             nextest_version: Version::new(0, 1, 0),
             started_at,
             last_written_at: started_at,
@@ -528,4 +592,109 @@ mod tests {
             "abc123"
         );
     }
+
+    #[test]
+    fn test_run_id_or_archive_selector_default() {
+        assert_eq!(
+            RunIdOrRecordingSelector::default(),
+            RunIdOrRecordingSelector::RunId(RunIdSelector::Latest)
+        );
+    }
+
+    #[test]
+    fn test_run_id_or_archive_selector_from_str() {
+        // "latest" parses to RunId(Latest).
+        assert_eq!(
+            "latest".parse::<RunIdOrRecordingSelector>().unwrap(),
+            RunIdOrRecordingSelector::RunId(RunIdSelector::Latest)
+        );
+
+        // Hex prefixes parse to RunId(Prefix(...)).
+        assert_eq!(
+            "abc123".parse::<RunIdOrRecordingSelector>().unwrap(),
+            RunIdOrRecordingSelector::RunId(RunIdSelector::Prefix("abc123".to_owned()))
+        );
+
+        // Paths ending in .zip parse to RecordingPath.
+        assert_eq!(
+            "nextest-run-abc123.zip"
+                .parse::<RunIdOrRecordingSelector>()
+                .unwrap(),
+            RunIdOrRecordingSelector::RecordingPath(Utf8PathBuf::from("nextest-run-abc123.zip"))
+        );
+        assert_eq!(
+            "/path/to/archive.zip"
+                .parse::<RunIdOrRecordingSelector>()
+                .unwrap(),
+            RunIdOrRecordingSelector::RecordingPath(Utf8PathBuf::from("/path/to/archive.zip"))
+        );
+        assert_eq!(
+            "../relative/path.zip"
+                .parse::<RunIdOrRecordingSelector>()
+                .unwrap(),
+            RunIdOrRecordingSelector::RecordingPath(Utf8PathBuf::from("../relative/path.zip"))
+        );
+
+        // Paths with separators (no .zip) parse to RecordingPath. This
+        // covers process substitution paths and relative paths.
+        assert_eq!(
+            "/proc/self/fd/11"
+                .parse::<RunIdOrRecordingSelector>()
+                .unwrap(),
+            RunIdOrRecordingSelector::RecordingPath(Utf8PathBuf::from("/proc/self/fd/11"))
+        );
+        assert_eq!(
+            "/dev/fd/5".parse::<RunIdOrRecordingSelector>().unwrap(),
+            RunIdOrRecordingSelector::RecordingPath(Utf8PathBuf::from("/dev/fd/5"))
+        );
+        assert_eq!(
+            "./my-recording"
+                .parse::<RunIdOrRecordingSelector>()
+                .unwrap(),
+            RunIdOrRecordingSelector::RecordingPath(Utf8PathBuf::from("./my-recording"))
+        );
+        assert_eq!(
+            "../path/to/file"
+                .parse::<RunIdOrRecordingSelector>()
+                .unwrap(),
+            RunIdOrRecordingSelector::RecordingPath(Utf8PathBuf::from("../path/to/file"))
+        );
+        // Backslash is treated as a path separator (Windows paths).
+        #[cfg(windows)]
+        {
+            assert_eq!(
+                r"C:\path\to\file"
+                    .parse::<RunIdOrRecordingSelector>()
+                    .unwrap(),
+                RunIdOrRecordingSelector::RecordingPath(Utf8PathBuf::from(r"C:\path\to\file"))
+            );
+        }
+
+        // Invalid run ID selector (not hex, not .zip, no path separator)
+        // still fails.
+        assert!("xyz".parse::<RunIdOrRecordingSelector>().is_err());
+        assert!("hello".parse::<RunIdOrRecordingSelector>().is_err());
+        // Typos of "latest" should still fail, not be treated as paths.
+        assert!("latets".parse::<RunIdOrRecordingSelector>().is_err());
+        assert!("latestt".parse::<RunIdOrRecordingSelector>().is_err());
+        // Bare filenames without separators or .zip extension should fail.
+        assert!("recording".parse::<RunIdOrRecordingSelector>().is_err());
+    }
+
+    #[test]
+    fn test_run_id_or_archive_selector_display() {
+        assert_eq!(
+            RunIdOrRecordingSelector::RunId(RunIdSelector::Latest).to_string(),
+            "latest"
+        );
+        assert_eq!(
+            RunIdOrRecordingSelector::RunId(RunIdSelector::Prefix("abc123".to_owned())).to_string(),
+            "abc123"
+        );
+        assert_eq!(
+            RunIdOrRecordingSelector::RecordingPath(Utf8PathBuf::from("/path/to/archive.zip"))
+                .to_string(),
+            "/path/to/archive.zip"
+        );
+    }
 }
