@@ -6,31 +6,72 @@
 //! The main structure in this module is [`TestReporter`].
 
 use super::{
-    FinalStatusLevel, MaxProgressRunning, StatusLevel, TestOutputDisplay,
-    displayer::{DisplayReporter, DisplayReporterBuilder, StatusLevels},
+    DisplayerKind, FinalStatusLevel, MaxProgressRunning, StatusLevel, TestOutputDisplay,
+    displayer::{DisplayReporter, DisplayReporterBuilder, ShowTerminalProgress, StatusLevels},
 };
 use crate::{
-    cargo_config::CargoConfigs,
     config::core::EvaluatableProfile,
     errors::WriteEventError,
     list::TestList,
+    record::{ShortestRunIdPrefix, StoreSizes},
     reporter::{
         aggregator::EventAggregator, displayer::ShowProgress, events::*,
         structured::StructuredReporter,
     },
+    write_str::WriteStr,
 };
+use std::time::Duration;
 
-/// Standard error destination for the reporter.
+/// Statistics returned by the reporter after a test run completes.
+#[derive(Clone, Debug, Default)]
+pub struct ReporterStats {
+    /// The sizes of the recording written to disk (compressed and uncompressed), or `None` if
+    /// recording was not enabled or an error occurred.
+    pub recording_sizes: Option<StoreSizes>,
+    /// Information captured from the `RunFinished` event.
+    pub run_finished: Option<RunFinishedInfo>,
+}
+
+/// Information captured from the `RunFinished` event.
 ///
-/// This is usually a terminal, but can be an in-memory buffer for tests.
-pub enum ReporterStderr<'a> {
-    /// Produce output on the (possibly piped) terminal.
+/// This struct groups together data that is always available together: if we
+/// receive a `RunFinished` event, we have both the stats and elapsed time.
+#[derive(Clone, Copy, Debug)]
+pub struct RunFinishedInfo {
+    /// Statistics about the run.
+    pub stats: RunFinishedStats,
+    /// Total elapsed time for the run.
+    pub elapsed: Duration,
+    /// The number of tests that were outstanding but not seen during this rerun.
+    ///
+    /// This is `None` if this was not a rerun. A value of `Some(0)` means all
+    /// outstanding tests from the rerun chain were seen during this run (and
+    /// either passed or failed).
+    pub outstanding_not_seen_count: Option<usize>,
+}
+
+/// Output destination for the reporter.
+///
+/// This is usually a terminal, but can be a writer for paged output or an
+/// in-memory buffer for tests.
+pub enum ReporterOutput<'a> {
+    /// Produce output on the terminal (stderr).
     ///
     /// If the terminal isn't piped, produce output to a progress bar.
     Terminal,
 
-    /// Write output to a buffer.
-    Buffer(&'a mut String),
+    /// Write output to a `WriteStr` implementation (e.g., for pager support or
+    /// an in-memory buffer for tests).
+    Writer {
+        /// The writer to use for output.
+        writer: &'a mut (dyn WriteStr + Send),
+        /// Whether to use unicode characters for output.
+        ///
+        /// The caller should determine this based on the actual output
+        /// destination (e.g., by checking `supports_unicode::on()` for the
+        /// appropriate stream).
+        use_unicode: bool,
+    },
 }
 
 /// Test reporter builder.
@@ -126,11 +167,11 @@ impl ReporterBuilder {
         &self,
         test_list: &TestList,
         profile: &EvaluatableProfile<'a>,
-        cargo_configs: &CargoConfigs,
-        output: ReporterStderr<'a>,
+        show_term_progress: ShowTerminalProgress,
+        output: ReporterOutput<'a>,
         structured_reporter: StructuredReporter<'a>,
     ) -> Reporter<'a> {
-        let aggregator = EventAggregator::new(profile);
+        let aggregator = EventAggregator::new(test_list.mode(), profile);
 
         let status_level = self.status_level.unwrap_or_else(|| profile.status_level());
         let final_status_level = self
@@ -138,6 +179,7 @@ impl ReporterBuilder {
             .unwrap_or_else(|| profile.final_status_level());
 
         let display_reporter = DisplayReporterBuilder {
+            mode: test_list.mode(),
             default_filter: profile.default_filter().clone(),
             status_levels: StatusLevels {
                 status_level,
@@ -152,13 +194,16 @@ impl ReporterBuilder {
             show_progress: self.show_progress,
             no_output_indent: self.no_output_indent,
             max_progress_running: self.max_progress_running,
+            show_term_progress,
+            displayer_kind: DisplayerKind::Live,
         }
-        .build(cargo_configs, output);
+        .build(output);
 
         Reporter {
             display_reporter,
             structured_reporter,
             metadata_reporter: aggregator,
+            run_finished: None,
         }
     }
 }
@@ -172,6 +217,8 @@ pub struct Reporter<'a> {
     metadata_reporter: EventAggregator<'a>,
     /// Used to emit test events in machine-readable format(s) to stdout
     structured_reporter: StructuredReporter<'a>,
+    /// Information captured from the RunFinished event.
+    run_finished: Option<RunFinishedInfo>,
 }
 
 impl<'a> Reporter<'a> {
@@ -187,8 +234,24 @@ impl<'a> Reporter<'a> {
     }
 
     /// Mark the reporter done.
-    pub fn finish(&mut self) {
+    ///
+    /// Returns statistics about the test run, including the size of the
+    /// recording if recording was enabled.
+    pub fn finish(mut self) -> ReporterStats {
         self.display_reporter.finish();
+        let recording_sizes = self.structured_reporter.finish();
+        ReporterStats {
+            recording_sizes,
+            run_finished: self.run_finished,
+        }
+    }
+
+    /// Sets the unique prefix for the run ID.
+    ///
+    /// This is used to highlight the unique prefix portion of the run ID
+    /// in the `RunStarted` output when a recording session is active.
+    pub fn set_run_id_unique_prefix(&mut self, prefix: ShortestRunIdPrefix) {
+        self.display_reporter.set_run_id_unique_prefix(prefix);
     }
 
     // ---
@@ -202,6 +265,21 @@ impl<'a> Reporter<'a> {
 
     /// Report this test event to the given writer.
     fn write_event(&mut self, event: Box<TestEvent<'a>>) -> Result<(), WriteEventError> {
+        // Capture run finished info before passing to reporters.
+        if let TestEventKind::RunFinished {
+            run_stats,
+            elapsed,
+            outstanding_not_seen,
+            ..
+        } = &event.kind
+        {
+            self.run_finished = Some(RunFinishedInfo {
+                stats: *run_stats,
+                elapsed: *elapsed,
+                outstanding_not_seen_count: outstanding_not_seen.as_ref().map(|t| t.total_not_seen),
+            });
+        }
+
         // TODO: write to all of these even if one of them fails?
         self.display_reporter.write_event(&event)?;
         self.structured_reporter.write_event(&event)?;
